use std::path::PathBuf;

use clap::Parser;
use clap_derive::Parser;
use svg_units::SVGMeasure;

#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    /// Input KLE JSON path
    input: PathBuf,
    /// Output SVG path
    #[arg(short, long)]
    output: Option<PathBuf>,
    /// Page width
    #[arg(long, default_value = "11in")]
    width: SVGMeasure,
    /// Page height
    #[arg(long, default_value = "8.5in")]
    height: SVGMeasure,
}

fn main() -> dyesub::Result<()> {
    let args = Args::parse();
    let output = args.output.unwrap_or_else(|| {
        let mut path = args.input.clone();
        path.set_extension("svg");
        path
    });
    dyesub::render_file(&args.input, output)?;
    Ok(())
}
