use std::error::Error;

use leptos::{html::Input, *};
use web_sys::File;

use super::ResultMessage;
use crate::{
    components::ResultMessageData, error::ReadKleError, models::Colorway, utils::read_file,
};

#[component]
pub fn KeyboardFromFile(
    set_keyboard: WriteSignal<Option<kle_serial::Keyboard>>,
) -> impl IntoView {
    let file_input = create_node_ref::<Input>();
    let (file, set_file) = create_signal(Option::<File>::None);
    let (result_message, set_result_message) = create_signal(Option::<ResultMessageData>::None);
    let on_submit = move || {
        spawn_local(async move {
            match read_kle_from_file(file).await {
                Ok(keyboard) => {
                    set_result_message(Some(ResultMessageData {
                        title: "Success".to_owned(),
                        message: format!(
                            "Loaded KLE layout <b>{}</b> successfully!",
                            &keyboard.metadata.name
                        ),
                        colorway: Colorway::Ok,
                    }));
                    set_keyboard(Some(keyboard));
                }
                Err(err) => {
                    set_result_message(Some({
                        let error: Box<dyn Error> = Box::new(err);
                        error.into()
                    }));
                    file_input().unwrap().set_value("");
                }
            }
        });
    };
    view! {
        <h3>"Load KLE JSON from file"</h3>
        <ResultMessage message=result_message />
        <input
            type="file"
            accept="application/json"
            node_ref=file_input
            on:change=move |_| {
                set_file(file_input().unwrap().files().and_then(|files| files.get(0)));
                on_submit();
            }
        />
    }
}

async fn read_kle_from_file(
    file: ReadSignal<Option<File>>,
) -> Result<kle_serial::Keyboard, ReadKleError> {
    Ok(serde_json::from_str(&read_file(file).await?)?)
}
