use std::error::Error;

use leptos::{html::Input, *};
use web_sys::SubmitEvent;

use super::{ResultMessage, ResultMessageData};
use crate::{error::FetchKleError, models::Colorway, utils::fetch_file};

async fn fetch_layout(url: &str) -> Result<kle_serial::Keyboard, FetchKleError> {
    let layout_string = fetch_file(url).await?;
    Ok(serde_json::from_str(&layout_string)?)
}

#[component]
pub fn KeyboardFromWeb(
    set_keyboard: WriteSignal<Option<kle_serial::Keyboard>>,
) -> impl IntoView {
    let (result_message, set_result_message) = create_signal(Option::<ResultMessageData>::None);
    let input_element: NodeRef<Input> = create_node_ref();
    let on_submit = move |e: SubmitEvent| {
        e.prevent_default();
        spawn_local(async move {
            let url = input_element.get().unwrap().value();
            match fetch_layout(&url).await {
                Ok(keyboard) => {
                    set_result_message(Some(ResultMessageData {
                        title: "Success".to_owned(),
                        message: format!(
                            "Loaded KLE layout <b>{}</b> successfully!",
                            &keyboard.metadata.name
                        ),
                        colorway: Colorway::Ok,
                    }));
                    set_keyboard(Some(keyboard));
                }
                Err(err) => {
                    set_result_message(Some({
                        let error: Box<dyn Error> = Box::new(err);
                        error.into()
                    }));
                    input_element().unwrap().set_value("");
                }
            }
        })
    };
    view! {
        <h3>"Load KLE JSON from web"</h3>
        <ResultMessage message=result_message />
        <form class="f-row align-items:center" on:submit=on_submit>
            <input class="width:100%" type="text" placeholder="KLE layout, gist URL, gist ID, or direct URL to JSON file" required node_ref=input_element />
            <input type="submit" value="Load" />
        </form>
    }
}
