use class_list::class_list;
use leptos::*;
use std::error::Error;

use crate::models::Colorway;

#[derive(Clone)]
pub struct ResultMessageData {
    pub title: String,
    pub message: String,
    pub colorway: Colorway,
}

impl From<Box<dyn Error>> for ResultMessageData {
    fn from(value: Box<dyn Error>) -> Self {
        Self {
            title: value.to_string(),
            message: value
                .source()
                .map(|source| source.to_string())
                .unwrap_or_default(),
            colorway: Colorway::Bad,
        }
    }
}

#[component]
pub fn ResultMessage(message: ReadSignal<Option<ResultMessageData>>) -> impl IntoView {
    let (open, set_open) = create_signal(true);
    create_effect(move |_| {
        message.track();
        set_open(true);
    });
    move || {
        view! {
            <Show when=open fallback=|| ()>
                {move || message().map(|ResultMessageData { title, message, colorway }| view! {
                    <div class=class_list!["box", colorway] style="position: relative">
                        <strong class="block titlebar">{title}</strong>
                        <button class="iconbutton" on:click=move |_| set_open(false) style="position: absolute; bottom: 0.5em; right: 0.5em">"×"</button>
                        <p inner_html=message />
                    </div>
                })}
            </Show>
        }
    }
}
